<?php
namespace AutomateYourTests\Controller;

/**
 * This class makes sure that a test <step> can run under a certain <user> context. Any configured <user> login
 * is executed upfront before the rest of the <testsuite> and a login cookie saved and re-used whenever needed.
 * a test <step> requires it.
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class LoginUser {
    /**
     * Class vars
     */
    protected $cookieFileName = 'AutomateYourTestsCookie{USER_LABEL}.tmp';
    protected $config = array();
    protected $model  = null;

    /**
     * Get set up
     */
    public function __construct($urlCaller)
    {
        $this->controller               = new \stdClass();
        $this->controller->urlCaller = $urlCaller;
    }

    /**
     * Login as our integration test user and let CURL remember the login cookie
     */
    public function logUserIn($userLoginStep = array())
    {
        $response = array();

        $userLabel = (string) $userLoginStep->attribute('label');

        // TODO: Protect against $this->controller->urlCaller->curlByUser[$userLabel] missing
        $curl = $this->controller->urlCaller->curlByUser[$userLabel];

        if (! in_array($userLabel, $this->getValidUserNames())) {
            throw new \Exception('A test has requested test login "' . $userLabel . '", but this login does not exist in the test configuration');
        } else {
            // Only login ONCE for each user
            // if (! isset($this->controller->urlCaller->cachedCookiePathByUser[$userLabel])) {
                $this->controller->urlCaller->cachedCookiePathByUser[$userLabel] = $this->getCookiePathByUser($userLabel);

                try {
                    // TODO: Offer this debug option
                    // \curl_setopt($curl, CURLOPT_VERBOSE, true);

                    // TODO: Catch and explain CURL 'Error: 3: <url> malformed' better .. and maybe other CURL errors too
                    \curl_setopt($curl, CURLOPT_URL, $userLoginStep->url);
                    \curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                    \curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true); // TODO: False, else it becomes impossible to differentiate 301 versus 200 statuses
                    \curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // Works with HTTPS without having to specify a .pem file
                    \curl_setopt($curl, CURLOPT_AUTOREFERER, true);

                    // Our programmatic login needs to set a login cookie (just like any normal login)
                    \curl_setopt($curl, CURLOPT_COOKIEJAR, $this->controller->urlCaller->cachedCookiePathByUser[$userLabel]); // writing cookies
                    \curl_setopt($curl, CURLOPT_COOKIEFILE, $this->controller->urlCaller->cachedCookiePathByUser[$userLabel]); // reading cookies

                    // Submit login credentials
                    if ($userLoginStep->post) {
                        \curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
                        \curl_setopt($curl, CURLOPT_POSTFIELDS, $userLoginStep->toString('post'));
                    }

                    // Allow applications to recognise requests coming from this testing app
                    //if ($this->config->useragent) {
                    //    \curl_setopt($curl, CURLOPT_USERAGENT, $this->config->useragent);
                    //}

                    $response['returnString'] = \curl_exec($curl);
                    $curlInfo = \curl_getinfo($curl);

                    $response['httpStatus'] = $curlInfo['http_code'];

                    $this->controller->urlCaller->curlByUser[$userLabel] = $curl;
                } catch (\Exception $exception) {
                    $response['httpStatus'] = -1;
                    $response['returnString'] = $exception->getCode() . ': ' . $exception->getMessage();
                }

                if ($response['returnString'] === false) {
                    $response['httpStatus'] = -1;
                    $response['returnString'] = 'HTTP request failed with "' . \curl_error($curl) .  ' (' . \curl_errno($curl) . ')" so please check your specified <url>' . $userLoginStep->url . '</url> or your Internet connection';
                }

                return $response;
            //}
        }
    }

    /**
     *
     */
    protected function getValidUserNames()
    {
        $validUserNames = array();

        foreach ($this->config->logins->user as $user) {
            $validUserNames[] = $user->attribute('label');
        }

        return $validUserNames;
    }

    /**
     *
     */
    public function getUserLoginStep($userLabel)
    {
        foreach ($this->config->logins->user as $user) {
            if ($user->attribute('label') == $userLabel) {
                return $user;
            }
        }

        return null;
    }

    /**
     * Return the cookie path for a specific login user
     */
    protected function getCookiePathByUser($userLabel)
    {
        if ($this->config->logins) {
            // Temp dir placeholder?
            if (strpos($this->config->logins->attribute('cookiedir'), '{TEMPDIR}') === false) {
                $cookieDir = $this->config->logins->attribute('cookiedir');
            } else {
                $cookieDir = str_replace('{TEMPDIR}', $this->getSystemTempDir(), $this->config->logins->attribute('cookiedir'));
            }

            // Safeguard against any file system non-sense
            if (! $cookieDir) {
                throw new \Exception('The cookiedir="" path has not been specified');
            } elseif (! \is_dir($cookieDir)) {
                throw new \Exception('The specified cookiedir="" path "' . $cookieDir . '" does not exist on disk (or your php.ini setting "open_basedir = ' . \ini_get('open_basedir') . '" path excludes it)');
            } elseif (! \is_writable($cookieDir)) {
                throw new \Exception('The specified cookiedir="" path "' . $cookieDir . '" is not writable');
            } elseif (! @\is_readable($cookieDir)) {
                throw new \Exception('The specified cookiedir="" path "' . $cookieDir . '" does not have the required READ permission (or your php.ini setting "open_basedir = ' . \ini_get('open_basedir') . '" path excludes it)');
            } else {
                //if ($this->config->logins->cookies->file) {
                    $sanitizedUserLabel = preg_replace("/[^a-zA-Z0-9]+/", "", $userLabel);
                    $substitutedCookieFilename = str_replace('{USER_LABEL}', $sanitizedUserLabel, $this->cookieFileName);

                    $cookiePath = $cookieDir . '/' . $substitutedCookieFilename;

                    return $cookiePath;
                //} else {
                //    throw new \Exception('The logins cookie file name must be specified');
                //}
            }
        }
    }

    /**
     * Get the system's temp directory
     */
    protected function getSystemTempDir()
    {
        // On some OS last trailing is missing/existing
        if (substr(sys_get_temp_dir(), -1) == '/') {
            $systemTempDir = substr(sys_get_temp_dir(), 0, strlen(sys_get_temp_dir()) - 1);
        } else {
            $systemTempDir = sys_get_temp_dir();
        }

        return $systemTempDir;
    }

    /**
     * Delete all user login cookies to start off with (else a wrong user login could not be corrected)
     */
    public function resetPreviousUserLoginCookie($userLabel)
    {
        $cookiePath = $this->getCookiePathByUser($userLabel);

        if (\file_exists($cookiePath)) {
            if (\is_writable($cookiePath)) {
                \unlink($cookiePath);
            }
        }
    }

    /**
     * Set config class var
     */
    public function setConfig($config = array())
    {
        $this->config = $config;
    }
}
