<?php
namespace AutomateYourTests\Controller;

/**
 * Use this class to RECORD any manual tests that you perform, so that you can PLAYBACK
 * the same tests automatically later using the AutomateYourTests testing software.
 *
 * Simply include & instantiate this class on your TEST server within your PHP
 * application's bootstrap (or base controller or any other place where it can
 * intercept HTTP requests). That script must, of course, have write access to
 * the directory of the 'myTestSuite.xml' that you specify:
 *
 * @example require_once('Controller/Recorder.php');
 *          new AutomateYourTests\Controller\Recorder('myTestSuite.xml');
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Recorder
{
    // TODO: Move template XML to here!
    protected $fileHandle;
    protected $filePath;
    protected $testSuiteConfigStart = '';
    protected $testSuiteConfigStep = '';
    protected $testSuiteConfigEnd = '';

    /**
     *
     */
    public function __construct($filePath)
    {
        $this->filePath = $filePath;

        $this->bufferHttpResponse();

        // Run AFTER the page response string has been generated
        register_shutdown_function(function() {
            $this->setTestSuiteConfigTemplates();
            $this->openTestSuiteFile();
            $this->buildTestStep();
            $this->closeTestSuiteFile();
        });
    }

    /**
     *
     */
    protected function openTestSuiteFile()
    {
        if ($this->checkFilePath()) {
            if (! file_exists($this->filePath)) {
                $startTemplate = $this->testSuiteConfigStart;
            } else {
                $startTemplate = '';
            }

            $this->fileHandle = fopen($this->filePath, 'a');

            if ($startTemplate) {
                chmod($this->filePath, 0777);
                fwrite($this->fileHandle, $startTemplate);
            }
        }
    }

    /**
     *
     */
    protected function buildTestStep()
    {
        $stepTemplate = $this->testSuiteConfigStep;

        // Replace <url> placeholder
        if (isset($_SERVER['REQUEST_URI'])) {
            $stepTitle = 'View URL ' . $_SERVER['REQUEST_URI']; // TODO: htmlentities()

            // Replace <post> placeholder (or remove it)
            if (isset($_POST) && count($_POST)) {
                if (strpos($_SERVER['REQUEST_URI'], '?') === false) {
                    $stepTitle .= '?';
                } else {
                    $stepTitle .= '&amp;';
                }

                $stepTitle .= urlencode(http_build_query($_POST)) . ' (POST data)';
            }

            $stepTemplate = str_replace('<step title="{STEP_TITLE_PLACEHOLDER}">', '<step title="' . $stepTitle . '">', $stepTemplate);
        }

        // Replace <url> placeholder
        if (isset($_SERVER['REQUEST_URI'])) {
            $stepTemplate = str_replace('<url>{URL_PLACEHOLDER}</url>', '<url>' . htmlentities($_SERVER['REQUEST_URI']) . '</url>', $stepTemplate);
        }

        // Replace <post> placeholder (or remove it)
        if (isset($_POST) && count($_POST)) {
            $stepTemplate = str_replace('<post>{POST_PLACEHOLDER}</post>', '<post><![CDATA[' . http_build_query($_POST) . ']]></post>', $stepTemplate);
        } else {
            $stepTemplate = str_replace('                <post>{POST_PLACEHOLDER}</post>' . "\n", '', $stepTemplate);
        }

        // Prepare <contains> placeholder
        //$now = \DateTime::createFromFormat('U.u', microtime(true));
        //$outputFileName = $now->format('Y_m_d') . '_at_' . $now->format('h_i_s_u') . '_http_response';
        //$this->saveHttpResponseString($outputFileName);

        // Replace <httpResponse> placeholder
        $responseString = $this->getHttpResponseString();
        $stepTemplate = str_replace("<!--<httpResponse>{RESPONSE_PLACEHOLDER}</httpResponse>-->", "<!--<httpResponse><![CDATA[" . $responseString . "]]></httpResponse>-->", $stepTemplate);

        $this->appendToTestSuite($stepTemplate);
    }

    /**
     *
     */
    /* protected function saveHttpResponseString($outputFileName)
    {
        $outputFilePath = dirname($this->filePath) . DIRECTORY_SEPARATOR . '' . $outputFileName . '.tmp';

        $responseString = $this->getHttpResponseString();

        $this->outputHandle = fopen($outputFilePath, 'w');
        fwrite($this->outputHandle, $responseString);
        fclose($this->outputHandle);
    } */

    /**
     *
     */
    protected function appendToTestSuite($substitutedTestStep)
    {
        $content = "\n" . $substitutedTestStep;

        fwrite($this->fileHandle, $content);
    }

    /**
     *
     */
    protected function closeTestSuiteFile()
    {
        fclose($this->fileHandle);
    }

    /**
     *
     */
    protected function setTestSuiteConfigTemplates()
    {
        $baseUrl = $this->getSiteUrl();

        // TODO: Put this into an extra template file !?
        $this->testSuiteConfigStart = <<<EOT
<testsuite>
    <inherit>
        <baseurl>$baseUrl</baseurl>
        <status>200</status>
        <status>301</status>
        <status>302</status>
        <notstatus>404</notstatus>
        <notstatus>500</notstatus>
    </inherit>
    <fixture>
        <setup label="Reset DB"><![CDATA[mysql --host=YOUR_DB_HOST --user=YOUR_DB_USER --password=YOUR_DB_PW YOUR_DB_NAME < YOUR_DB_SQL_PATH.sql]]></setup>
    </fixture>
    <logins title="Let each test step execute in the context of an authenticated user" cookiedir="{TEMPDIR}">
        <user label="COMPLETE_OR_REMOVE">
            <url>COMPLETE_OR_REMOVE</url>
            <post><![CDATA[COMPLETE_OR_REMOVE]]></post>
        </user>
    </logins>
    <userstory title="COMPLETE_OR_REMOVE">
        <scenario title="COMPLETE_OR_REMOVE">
            <step title="COMPLETE_OR_REMOVE">
                <setup>Reset DB</setup>
            </step>
EOT;
        $this->testSuiteConfigStep = <<<EOT
            <step title="{STEP_TITLE_PLACEHOLDER}">
                <!-- 1. FIRST EVALUATE THE HTTP RESPONSE AT THE BOTTOM OF THIS STEP AS TO WHAT YOU WANT YOUR TEST ASSERTION(S) TO BE -->
                <!-- 2. SECONDLY FILL IN EITHER OF THE FOLLOWING TEST ASSERTIONS AND REMOVE THE COMMENT THAT SURROUNDS IT -->
                <!-- 3. AT THE VERY END WHEN YOUR WHOLE TEST SUITE PASSES YOU THEN FINALLY REMOVE ALL THE FOLLOWING COMMENTED OUT XML TAGS THAT ARE STILL LEFT -->
                <!--<user>COMPLETE_OR_REMOVE</user>-->
                <url>{URL_PLACEHOLDER}</url>
                <post>{POST_PLACEHOLDER}</post>
                <!--<contains>COMPLETE_OR_REMOVE</contains>-->
                <!--<notcontains>COMPLETE_OR_REMOVE</notcontains>-->
                <!--<status>COMPLETE_OR_REMOVE</status>-->
                <!--<notstatus>COMPLETE_OR_REMOVE</notstatus>-->
                <!--<xpath>COMPLETE_OR_REMOVE</xpath>-->
                <!--<httpResponse>{RESPONSE_PLACEHOLDER}</httpResponse>-->
            </step>
EOT;
        // TODO: Append this $this->testSuiteConfigEnd, and then insert steps into the middle
        $this->testSuiteConfigEnd = <<<EOT
        </scenario>
    </userstory>
</testsuite>
EOT;
    }

    /**
     *
     */
    protected function checkFilePath() {
        if (! $this->filePath) {
            throw new \Exception("When instantiating AutomateYourTestsRecorder('myTestSuite.xml') you must pass a myTestSuite.xml file path where you want the test suite to be created");
        //} elseif (file_exists($this->filePath)) {
        //    throw new \Exception('The file path "' . $this->filePath . '" already exists');
        } elseif (! @is_writable(dirname($this->filePath))) {
            throw new \Exception('The file path "' . realpath(dirname($this->filePath)) . '" does not have WRITE permission for this user (or your php.ini setting "open_basedir = ' . \ini_get('open_basedir') . ' does not allow writing to it)');
        }

        return true;
    }

    /**
     * TODO: Check, if output buffering is enabled, i.e. output_buffering = off/0 and ob_get_level()
     */
    protected function bufferHttpResponse()
    {
        ob_start();
    }

    /**
     *
     */
    protected function getHttpResponseString()
    {
        return ob_get_contents();
    }

    /**
     *
     */
    protected function getSiteUrl() {
        $protocol = (! empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
        $domainName = $_SERVER['HTTP_HOST'];

        return $protocol . $domainName;
    }
}
