<?php
namespace AutomateYourTests\Controller;

/**
 * This class makes the remote HTTP connection for every test <step> and returns its result
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class UrlCaller {
    /**
     * Class vars
     */
    protected $config               = array();
    // TODO: Make all these protected again
    public $curlByUser              = array();
    public $cachedCookiePathByUser  = array();

    /**
     * Call the test URL and return response
     */
    public function makeHttpRequest($step)
    {
        $response = array();

        try {
            $userLabel = (string) $step->toString('user');

            // Any user label specified?
            if ($userLabel && isset($this->curlByUser[$userLabel])) {
                // TODO: Protect against $this->controller->urlCaller->openCurlConnectionByUser(); call missing
                $curl = $this->curlByUser[$userLabel];
            } else {
                $curl = $this->curlByUser[''];
            }

            // TODO: Offer this 'verbose' option
            // \curl_setopt($curl, CURLOPT_VERBOSE, true);

            // TODO: Catch and explain CURL 'Error: 3: <url> malformed' better .. and maybe other CURL errors too
            \curl_setopt($curl, CURLOPT_URL, $step->url);
            \curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            \curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true); // TODO: False, else it becomes impossible to differentiate 301 versus 200 statuses in the tests
            \curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // Works with HTTPS without having to specify a .pem file
            \curl_setopt($curl, CURLOPT_AUTOREFERER, true);

            if ($step->post) {
                \curl_setopt($curl, CURLOPT_POST, true); // This sets the http header content-length, which some web clients require
                \curl_setopt($curl, CURLOPT_POSTFIELDS, $step->toString('post'));
            }

            // Any user label specified?
            if ($userLabel && isset($this->curlByUser[$userLabel])) {
                // TODO: if (\is_readable($this->cachedCookiePathByUser[$userLabel])) {
                if ($this->config->logins->attribute('cookiedir')) {
                    // Our programmatic login needs a login cookie (just like any normal login)
                    \curl_setopt($curl, CURLOPT_COOKIEJAR, $this->cachedCookiePathByUser[$userLabel]); // writing cookies
                    \curl_setopt($curl, CURLOPT_COOKIEFILE, $this->cachedCookiePathByUser[$userLabel]); // reading cookies
                } else {
                    $response['httpStatus'] = -1;
                    $response['returnString'] = 'Could not read login cookie "' . $this->cachedCookiePathByUser[$userLabel] . '" for user "' . $userLabel . '" from configured cookie directory "' . $this->config->logins->cookies->dir . '"';

                    return $response;
                }
            }

            // TODO: Allow applications to recognise requests coming from this testing app
            //if ($this->config->useragent) {
            //    \curl_setopt($curl, CURLOPT_USERAGENT, $this->config->useragent);
            //}

            // TODO: Allow to inspect/test for HTTP header content
            //\curl_setopt($curl, CURLOPT_HEADER, true);

            $response['returnString'] = \curl_exec($curl);

            $curlInfo = \curl_getinfo($curl);
            $response['httpStatus'] = $curlInfo['http_code'];

            // TODO: Allow to inspect/test for HTTP header content
            //preg_match_all('/^Set-Cookie:\s*([^;]*)/mi', $response['returnString'], $matches);
            //$cookies = array();
            //foreach($matches[1] as $item) {
            //    parse_str($item, $cookie);
            //    $cookies = array_merge($cookies, $cookie);
            //}
            //var_dump($response['returnString']);

            // TODO: Honour $curlInfo['content_type'] = application/json?

            // TODO: Needed?
            //if ($userLabel && isset($this->curlByUser[$userLabel])) { // user label may not have resulted in any CURL connection
            //    $this->curlByUser[$userLabel] = $curl;
            //} else {
            //    $this->curlByUser[''] = $curl;
            //}
        } catch (\Exception $exception) {
            $response['httpStatus'] = -1;
            $response['returnString'] = $exception->getCode() . ': ' . $exception->getMessage();
        }

        if ($response['returnString'] === false) {
            $response['httpStatus'] = -1;
            $response['returnString'] = 'HTTP request failed with "' . \curl_error($curl) .  ' (' . \curl_errno($curl) . ')" so please check your specified <url>' . $step->url . '</url> or your Internet connection';
        }

        return $response;
    }

    /**
     * Open CURL connection
     */
    public function openCurlConnectionByUser($userLoginStep = array())
    {
        // Better safe than sorry
        // TODO: Move to Environmentor() class
        if (! \extension_loaded('curl') || ! \function_exists('curl_init')) {
            throw new \Exception('The PHP extension "CURL" is required, but your current PHP installation has not got it installed or enabled. Please find installation instructions at http://automateyourtests.com/how-to-install');
        } else {
            $curlInit = \curl_init();
        }

        if ($userLoginStep) {
            $userLabel = $userLoginStep->attribute('label');

            // If not exists, just in case
            if (! isset($this->curlByUser[$userLabel])) {
                $this->curlByUser[$userLabel] = $curlInit;

                if (\curl_errno($this->curlByUser[$userLabel])) {
                    throw new \Exception(\curl_errno($this->curlByUser[$userLabel]) . ': ' . \curl_error($this->curlByUser[$userLabel]));
                }
            }
        } else {
            $this->curlByUser[''] = $curlInit;
        }
    }

    /**
     * Close CURL connection
     */
    public function closeAllCurlConnections()
    {
        foreach ((array) $this->curlByUser as $curlByUser) {
            curl_close($curlByUser);
        }
    }

    /**
     * Set config class var
     */
    public function setConfig($config = array())
    {
        $this->config = $config;
    }
}
