<?php
namespace AutomateYourTests\Model\Config;

/**
 * This class contains a PHP array with ALL the valid XML test suite syntax in it.
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Base {
    /**
     * Class vars
     */
    protected $validXmlStructure = array(
        'testsuite' => array(
            'required' => true, // WARNING: This must never be user-defined, cos this is the XML root
            'occurance' => 'scalar', // WARNING: This must never be user-defined, cos this is the XML root
            'children' => array(
                'inherit' => array(
                    'required' => false,
                    'occurance' => 'array',
                    'children' => array(
                        'baseurl' => array(
                            'required' => false,
                            'occurance' => 'scalar',
                            'children' => 'string'
                        ),
                        'url' => array(
                            'required' => false,
                            'occurance' => 'scalar',
                            'children' => 'string'
                        ),
                        'user' => array(
                            'required' => false,
                            'occurance' => 'scalar',
                            'children' => 'string'
                        ),
                        'post' => array(
                            'required' => false,
                            'occurance' => 'scalar',
                            'children' => 'string'
                        ),
                        'contains' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string'
                        ),
                        'notcontains' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string'
                        ),
                        'xpath' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string',
                        ),
                        'status' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'integer'
                        ),
                        'notstatus' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'integer'
                        ),
                    ),
                    'attributes' => array(
                        'title' => array(
                            'required' => false,
                        ),
                    ),
                ),
                'fixture' => array(
                    'required' => false,
                    'occurance' => 'scalar',
                    'children' => array(
                        'setup' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string',
                            'attributes' => array(
                                'label' => array(
                                    'required' => true,
                                ),
                            ),
                        ),
                        'teardown' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string',
                            'attributes' => array(
                                'label' => array(
                                    'required' => true,
                                ),
                            ),
                        ),
                    ),
                ),
                'setup' => array(
                    'required' => false,
                    'occurance' => 'array',
                    'children' => 'string'
                ),
                'logins' => array(
                    'required' => false,
                    'occurance' => 'scalar',
                    'children' => array(
                        'inherit' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => array(
                                'baseurl' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'url' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'user' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'post' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'contains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'notcontains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'xpath' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string',
                                ),
                                'status' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                                'notstatus' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                            ),
                            'attributes' => array(
                                'title' => array(
                                    'required' => false,
                                ),
                            ),
                        ),
                        'user' => array(
                            'required' => true,
                            'occurance' => 'array',
                            'children' => array(
                                'setup' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'url' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'post' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'contains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'notcontains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'xpath' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string',
                                ),
                                'status' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                                'notstatus' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                                'teardown' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                            ),
                            'attributes' => array(
                                'label' => array(
                                    'required' => true,
                                ),
                            ),
                        ),
                    ),
                    'attributes' => array(
                        'title' => array(
                            'required' => false,
                        ),
                        'cookiedir' => array(
                            'required' => false, // TODO: Dependant validation (cos <logins> is not required)
                        ),
                    ),
                ),
                'userstory' => array(
                    'required' => true,
                    'occurance' => 'array',
                    'children' => array(
                        'inherit' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => array(
                                'baseurl' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'url' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'user' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'post' => array(
                                    'required' => false,
                                    'occurance' => 'scalar',
                                    'children' => 'string'
                                ),
                                'contains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'notcontains' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'xpath' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string',
                                ),
                                'status' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                                'notstatus' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'integer'
                                ),
                            ),
                            'attributes' => array(
                                'title' => array(
                                    'required' => false,
                                ),
                            ),
                        ),
                        'setup' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string'
                        ),
                        'scenario' => array(
                            'required' => true,
                            'occurance' => 'array',
                            'children' => array(
                                'inherit' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => array(
                                        'baseurl' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'url' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'user' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'post' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'contains' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                        'notcontains' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                        'xpath' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string',
                                        ),
                                        'status' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'integer'
                                        ),
                                        'notstatus' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'integer'
                                        ),
                                    ),
                                    'attributes' => array(
                                        'title' => array(
                                            'required' => false,
                                        ),
                                    ),
                                ),
                                'setup' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                                'step' => array(
                                    'required' => true,
                                    'occurance' => 'array',
                                    'children' => array(
                                        'baseurl' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'setup' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                        'url' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'user' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'post' => array(
                                            'required' => false,
                                            'occurance' => 'scalar',
                                            'children' => 'string'
                                        ),
                                        'contains' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                        'notcontains' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                        'xpath' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string',
                                        ),
                                        'status' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'integer'
                                        ),
                                        'notstatus' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'integer'
                                        ),
                                        'teardown' => array(
                                            'required' => false,
                                            'occurance' => 'array',
                                            'children' => 'string'
                                        ),
                                    ),
                                    'attributes' => array(
                                        'title' => array(
                                            'required' => false,
                                        ),
                                    ),
                                ),
                                'teardown' => array(
                                    'required' => false,
                                    'occurance' => 'array',
                                    'children' => 'string'
                                ),
                            ),
                            'attributes' => array(
                                'title' => array(
                                    'required' => false,
                                ),
                            ),
                        ),
                        'teardown' => array(
                            'required' => false,
                            'occurance' => 'array',
                            'children' => 'string'
                        ),
                    ),
                    'attributes' => array(
                        'title' => array(
                            'required' => false,
                        ),
                    ),
                ),
                'teardown' => array(
                    'required' => false,
                    'occurance' => 'array',
                    'children' => 'string'
                ),
            ),
        ),
    );
}
