<?php
namespace AutomateYourTests\Model\Config;

/**
 * This class checks the environment that the testing software runs in. It throws exceptions,
 * if any problems are encountered that stop the software from functioning correctly.
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Environmentor extends Base {
    /**
     * Check if the environment is friendly towards running this
     * software, eg php.ini settings, loaded extensions, etc.
     */
    public function checkSystemRequirements()
    {
        $this->isCurlEnabled();
        $this->isSimpleXmlEnabled();
    }

    /**
     *
     */
    protected function isCurlEnabled()
    {
        if (! \extension_loaded('curl') || ! \function_exists('curl_init')) {
            throw new \Exception('The PHP extension "CURL" is required, but your current PHP installation has not got it installed or enabled. Please find installation instructions at http://automateyourtests.com/how-to-install');
            //return false;
        } elseif ($this->isFunctionDisabled('curl_init')) {
            throw new \Exception('Your php.ini file lists "curl_init" in "disable_functions", which must be enabled for this software to work');
            //return false;
        //} else {
        //    return true;
        }
    }

    /**
     *
     */
    protected function isSimpleXmlEnabled()
    {
        if (! \extension_loaded('simplexml') || ! \class_exists('SimpleXMLElement')) {
            throw new \Exception('The PHP extension "SimpleXML" is required, but your current PHP installation has not got it installed or enabled. Please find installation instructions at http://automateyourtests.com/how-to-install');
            //return false;
        } elseif (! class_exists('DOMDocument')) {
            throw new \Exception('The PHP extension "DOMDocument" is required, but your current PHP installation has not got it installed or enabled. Please find installation instructions at http://automateyourtests.com/how-to-install');
            //return false;
        } elseif ($this->isClassDisabled('SimpleXMLElement')) {
            throw new \Exception('Your php.ini file lists "SimpleXMLElement" in "disable_classes", which must be enabled for this software to work');
            //return false;
        } elseif ($this->isClassDisabled('DOMDocument')) {
            throw new \Exception('Your php.ini file lists "DOMDocument" in "disable_classes", which must be enabled for this software to work');
            //return false;
        //} else {
        //    return true;
        }
    }

    /**
     *
     */
    public function checkExec()
    {
        if ($this->isFunctionDisabled('exec')) {
            throw new \Exception('Your php.ini file lists "exec" in "disable_functions", but your test config contains <setup> and/or <teardown> nodes, which rely on the "exec" function');
        }
    }

    /**
     *
     */
    protected function isClassDisabled($classToCheck)
    {
        $disabledClasses = explode(',', \ini_get('disable_classes'));

        if ($disabledClasses) {
            foreach ((array) $disabledClasses as $disabledClass) {
                if (\trim($disabledClass) == $classToCheck) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     *
     */
    protected function isFunctionDisabled($functionToCheck)
    {
        $disabledFunctions = explode(',', \ini_get('disable_functions'));

        if ($disabledFunctions) {
            foreach ((array) $disabledFunctions as $disabledFunction) {
                if (\trim($disabledFunction) == $functionToCheck) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     *
     */
    //public function checkSafeMode()
    //{
    //    if (\ini_get('safe_mode')) {
    //        throw new \Exception('Your php.ini settings file has "safe_mode" enabled, but this software requires it to be disabled');
    //    }
    //}
}
