<?php
namespace AutomateYourTests\Model\Config;

/**
 * This class is a slight extension to the popular SimpleXMLElement PHP library that we use to parse the test suite's XML.
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class ExtendedXMLElement extends \SimpleXMLElement {
    /**
     * Enable enhanced XML functionality, eg xinclude
     */
    public function enhanceXmlParsing()
    {
        //try {
            // Allow <![CDATA[]]> content
            $libXmlOptions = LIBXML_NOCDATA;
            //$libXmlOptions = LIBXML_NOERROR;
            //$libXmlOptions = LIBXML_XINCLUDE;

            $dom = \dom_import_simplexml($this);
            $includeCount = $dom->ownerDocument->xinclude($libXmlOptions);

            //$dom = new \DOMDocument;
            //$dom->loadXML($xmlConfig);

            if ($includeCount == -1) {
                throw new \Exception('The xinclude operation has failed on this XML file');
            }
        //} catch (\Exception $e) {
        //    exit("Error: " . $e->getMessage() . "\n");
        //}
    }

    /**
     * Returns a SimpleXMLElement node object (inner text) as string type
     */
    public function toString($nodeName) {
        return (string) $this->$nodeName;
    }

    /**
     * Returns a SimpleXMLElement node object as array type
     */
    public function toArray($nodeName) {
        return (array) $this->$nodeName;
    }

    /**
     *
     */
    public function attribute($attributeName) {
        $attributes = $this->attributes();

        // Else attribute would be of SimpleXMLElement object type
        if (isset($attributes->$attributeName)) {
            return (string) $attributes->$attributeName;
        } else {
            return '';
        }
    }

    /**
     *
     */
    public function hasAttribute($attributeName) {
        return (boolean) $this->attribute($attributeName);
    }

    /**
     * Convert PHP array into SimpleXMLElement
     *
     * @return SimpleXMLElement
     */
    static public function fromArray($template_info, $xml = false)
    {
        if ($xml === false){
            $xml = new \SimpleXMLElement('<?xml version="1.0"?><jsonToXml></jsonToXml>');
        }

        foreach ($template_info as $key => $value) {
            if (is_array($value)) {
                if (! is_numeric($key)) {
                    self::fromArray($value, $xml->addChild($key));
                } else{
                    self::fromArray($value, $xml);
                }
            } else {
                $xml->addChild($key,$value);
            }
        }

        return $xml;
    }
}
