<?php
namespace AutomateYourTests\View;

/**
 * This class contains all the logging output resulting from the running of a test suite.
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

class Logger {
    /**
     * Config vars
     */
    protected $unknownLineNoSymbol = '?';

    /**
     * Class vars
     */
    // TODO: Make all these protected again
    public $allCount                  = 0;
    public $failureCount              = 0;
    protected $log                    = '';
    protected $partialLog             = array();
    protected $fileNames              = array();
    protected $lastFileNo             = 0;
    protected $lastLineNo             = 0;
    protected $lastLineFraction       = 0.0001;
    protected $model                  = null;
    protected $config                 = array();
    protected $scriptStartDate        = null;
    protected $scriptEndDate          = null;
    protected $extraLogInfo           = '';

    /**
     * Get set up
     */
    public function __construct($counter, $fixture)
    {
        $this->model          = new \stdClass();
        $this->model->counter = $counter;
        $this->model->fixture = $fixture;
    }

    /**
     * Format the output of the separator
     */
    protected function separator($before = 1, $after = 2)
    {
        $separator  = \str_repeat("\n", $before);
        $separator .= '---------------------------------------------------------------------------------------------------';
        $separator .= \str_repeat("\n", $after);

        return $separator;
    }

    /**
     *
     */
    public function addSeparator($before = 1, $after = 2)
    {
        $sort = $this->getNextSort();

        $this->partialLog[$sort] = $this->separator($before, $after);
    }

    /**
     * Format the output of a user story, a scenario, or test step
     * that is NOT at the bottom of the XML hierarchy
     */
    public function testSuiteTitle($testsuiteNode)
    {
        $sort = $this->getSort($testsuiteNode);
        $lineNo = $this->getLineNo($sort);
        //$sort = $this->getNextSort();

        // Remember start time of script
        $this->scriptStartDate = new \DateTime();
        $prettyStartDate = $this->scriptStartDate->format('D m/d/Y H:i:s');
        $prettyStartDate = strtoupper($prettyStartDate);

        //$this->partialLog[$sort]  = $this->separator();
        $this->partialLog[$sort] = "\n" . $lineNo . ': <testsuite>: Starts running ' . strtolower($prettyStartDate) . "\n";
        //$this->partialLog[$sort] = "\n" . 'TEST SUITE ' . \realpath($testsuite) . ' STARTED RUNNING ' . $prettyStartDate . "\n";
        //$this->partialLog[$sort] = "\n" . 'TEST SUITE 1_hello_world.xml STARTED RUNNING ' . $prettyStartDate . "\n";
    }

    /**
     * The <logins> heading for all the upfront user logins
     */
    public function logins($loginsNode)
    {
        $sort = $this->getSort($loginsNode);
        $lineNo = $this->getLineNo($sort);

        $this->partialLog[$sort]  = $this->separator();
        $this->partialLog[$sort] .= $lineNo . ': <logins>: ' . $loginsNode->attribute('title') . "\n";
    }

    /**
     * Log the <user label=""> of an <logins>
     */
    public function loginUser($userCount, $userNode)
    {
        $sort = $this->getSort($userNode);
        $lineNo = $this->getLineNo($sort);

        $this->partialLog[$sort]  = $this->separator();

        if ($userNode) {
            $userLabel = $userNode->attribute('label');
        } else {
            $userLabel = '';
        }

        $this->partialLog[$sort] .= $lineNo . ': <user>: ' . ($userCount + 1) . ': ' . $userLabel . "\n";
    }

    /**
     * The <logins> heading for all the upfront user logins
     */
    public function userStoriesStart($levels)
    {
        $sort = $this->getSort($levels);

        $this->partialLog[$sort]  = $this->separator();
        $this->partialLog[$sort] .= "USER STORIES, THEIR TEST SCENARIOS AND STEPS:\n";
        // TODO: $this->partialLog[$sort] .= $this->getUserStoryCount() . " USER STORIES, THEIR TEST SCENARIOS AND STEPS AS FOLLOW\n";
    }

    /**
     * Format the output of a user story, a scenario, or test step
     * that is NOT at the bottom of the XML hierarchy
     */
    public function topNode($fieldName = '', $elementCount = 0, $upperNode = array())
    {
        $sort = $this->getSort($upperNode);
        $lineNo = $this->getLineNo($sort);

        $this->partialLog[$sort] = $this->separator();

        // Title is optional
        // TODO: Make 'userstory' convert into 'USER STORY', not 'userstory'
        if ($upperNode->attribute('title')) {
            $this->partialLog[$sort] .= $lineNo . ': <' . $fieldName . '> ' . $elementCount . ': ' . $upperNode->attribute('title') . "\n";
        } else {
            $this->partialLog[$sort] .= $lineNo . ': <' . $fieldName . '> ' . $elementCount . "\n";
        }
    }

    /**
     * Format the output of a user story, a scenario, or test step
     * that IS AT THE bottom of the XML hierarchy
     */
    public function bottomNode($fieldName = '', $elementCount = 0, $step = array())
    {
        $sort = $this->getSort($step);
        $lineNo = $this->getLineNo($sort);

        $this->partialLog[$sort] = $this->separator();

        // Title is optional
        if ($step->attribute('title')) {
            $this->partialLog[$sort] .= $lineNo . ': <' . $fieldName . '> ' . $elementCount . ': ' . $step->attribute('title') . "\n";
        } else {
            $this->partialLog[$sort] .= $lineNo . ': <' . $fieldName . '> ' . $elementCount . "\n";
        }
    }

    /**
     * Log the <user> of a <step>
     */
    public function user($step = array())
    {
        if ($step->user) {
            $sort = $this->getSort($step->user);
            $lineNo = $this->getLineNo($sort);

            $this->partialLog[$sort] = $lineNo . ': <user>: ' . $step->user . "\n";
        }
    }

    /**
     * Log the <url> of a <step>
     */
    public function url($step = array(), $response = array())
    {
        $sort = $this->getSort($step->url);
        $lineNo = $this->getLineNo($sort);

        // HTTP request failed fundamentally
        if ($response['httpStatus'] == -1) {
            $httpFailureString = ' (HTTP request failed) (FAILURE)';
        } else {
            $httpFailureString = '';
        }

        $this->partialLog[$sort] = $lineNo . ': <url>: ' . $step->url . $httpFailureString . "\n";
    }

    /**
     * Log an optional <post>> of a <step>
     */
    public function post($step = array(), $response = array())
    {
        if ($step->post) {
            $sort = $this->getSort($step->post);
            $lineNo = $this->getLineNo($sort);

            // HTTP request failed fundamentally
            if ($response['httpStatus'] == -1) {
                $httpFailureString = ' (HTTP request failed) (FAILURE)';
            } else {
                $httpFailureString = '';
            }

            $this->partialLog[$sort] = $lineNo . ': <post>: ' . $step->post . $httpFailureString . "\n";
        }
    }

    /**
     * Format an (optional) set up task
     */
    public function setup($setupLabel, $setupNode, $errorString = '')
    {
        $sort = $this->getSort($setupLabel);
        $lineNo = $this->getLineNo($sort);

        // Log just next behind the parent node
        $setupSort = $this->getSort($setupNode);
        //$setupSort = $this->getNextSort();

        // Be lenient
        $trimmedSetupLabel = trim($setupLabel);

        if (! $errorString) {
            $this->partialLog[$setupSort] = $lineNo . ': <setup>: Setup label "' . $trimmedSetupLabel . '" resulted in exit code 0 (PASS)' . "\n";
        } else {
            $this->partialLog[$setupSort] = $lineNo . ': <setup>: Setup label "' . $trimmedSetupLabel . '" resulted in exit code 1 (FAILURE)' . "\n";
        }
    }

    /**
     * Format an (optional) tear down task
     */
    public function teardown($teardownLabel, $teardownNode, $errorString = '')
    {
        $sort = $this->getSort($teardownLabel);
        $lineNo = $this->getLineNo($sort);

        // Log just next behind the parent node
        $teardownSort = $this->getSort($teardownNode);
        //$teardownSort = $this->getNextSort();

        // Be lenient
        $trimmedTeardownLabel = trim($teardownLabel);

        if (! $errorString) {
            $this->partialLog[$teardownSort] = $lineNo . ': <teardown>: Teardown label "' . $trimmedTeardownLabel . '" resulted in exit code 0 (PASS)' . "\n";
        } else {
            $this->partialLog[$teardownSort] = $lineNo . ': <teardown>: Teardown label "' . $trimmedTeardownLabel . '" resulted in exit code 1 (FAILURE)' . "\n";
        }
    }

    /**
     * Log string containment result
     */
    public function contains($isFailure, $response, $containNode)
    {
        $sort = $this->getSort($containNode);
        $lineNo = $this->getLineNo($sort);

        if ($isFailure === 0) {
            $this->partialLog[$sort] = $lineNo . ': <contains>: ' . $containNode . ' (PASS)' . "\n";
        } else {
            $this->partialLog[$sort] = $lineNo . ': <contains>: ' . $containNode . ' (FAILURE)'. "\n";
        }
    }

    /**
     * Log string containment result of a negative (not) comparison
     */
    public function notcontains($isFailure, $response, $notContainNode)
    {
        $sort = $this->getSort($notContainNode);
        $lineNo = $this->getLineNo($sort);

        if ($isFailure === 0) {
            $this->partialLog[$sort] = $lineNo . ': <notcontains>: ' . $notContainNode . ' (PASS)' . "\n";
        } else {
            $this->partialLog[$sort] = $lineNo . ': <notcontains>: ' . $notContainNode . ' (FAILURE)'. "\n";
        }
    }

    /**
     * Log JSON xpath result
     */
    public function xpath($isFailure, $response, $xpathNode)
    {
        $sort = $this->getSort($xpathNode);
        $lineNo = $this->getLineNo($sort);

        // TODO: Matches what exact content (for debugging)?
        if ($isFailure === 0) {
            $this->partialLog[$sort] = $lineNo . ': <xpath>: ' . $xpathNode . ' (PASS)' . "\n";
        } else {
            $this->partialLog[$sort] = $lineNo . ': <xpath>: ' . $xpathNode . ' (FAILURE)' . "\n";
        }
    }

    /**
     * Log HTTP Status result
     */
    public function status($isFailure, $response, $actualHttpStatus, $expectedStatusNode)
    {
        // Singular wording?
        /* if (count($expectedStatusNode) == 1) {
            $codesWording = 'code';
            $containValueWording = 'is equal';
            $notContainWording = 'is not equal';
        } else {
            $codesWording = 'codes';
            $containValueWording = 'contain';
            $notContainWording = 'do not contain';
        } */

        $sort = $this->getSort($expectedStatusNode);
        $lineNo = $this->getLineNo($sort);

        // SimpleXmlObject to array
        $expectedHttpStatusArrays = (array) $expectedStatusNode;

        if ($isFailure === 0) {
            $this->partialLog[$sort] = $lineNo . ': <status>: ' . implode(', ', $expectedHttpStatusArrays) . ' (actual HTTP status code ' . $actualHttpStatus . ') (PASS)' . "\n";
        } else {
            $this->partialLog[$sort] = $lineNo . ': <status>: ' . implode(', ', $expectedHttpStatusArrays) . ' (actual HTTP status code ' . $actualHttpStatus . ') (FAILURE)' . "\n";
        }
    }

    /**
     * Log HTTP Status result of a negative (not) comparison
     */
    public function notstatus($isFailure, $response, $actualHttpStatus, $notExpectedStatusNode)
    {
        // Singular wording?
        /* if (count($notExpectedStatusNode) == 1) {
            $codesWording = 'code';
            $containWording = 'is equal';
            $notContainWording = 'is not equal';
        } else {
            $codesWording = 'codes';
            $containWording = 'contain';
            $notContainWording = 'do not contain';
        } */

        $sort = $this->getSort($notExpectedStatusNode);
        $lineNo = $this->getLineNo($sort);

        // SimpleXmlObject to array
        $notExpectedHttpStatusArrays = (array) $notExpectedStatusNode;

        if ($isFailure === 0) {
            $this->partialLog[$sort] = $lineNo . ': <notstatus>: ' . implode(', ', $notExpectedHttpStatusArrays) . ' (actual HTTP status code ' . $actualHttpStatus . ') (PASS)' . "\n";
        } else {
            $this->partialLog[$sort] = $lineNo . ': <notstatus>: ' . implode(', ', $notExpectedHttpStatusArrays) . ' (actual HTTP status code ' . $actualHttpStatus . ') (FAILURE)'. "\n";
        }
    }

    /**
     *
     */
    public function logPassOrFail($isFailure)
    {
        $sort = $this->getNextSort(1); // TODO: Get rid of this dirty hack!

        if ($isFailure === 0) {
            $this->partialLog[$sort] = "\n" . 'PASS' . "\n";
        } else {
            $this->partialLog[$sort] = "\n" . 'FAILURE' . "\n";
        }
    }


    /**
     * Give ultra-verbose feedback containing HTTP response ?
     */
    public function verboseHttpResponse($httpResponse)
    {
        $sort = $this->getNextSort();

        if (isset($httpResponse['httpStatus'])) {
            $this->partialLog[$sort] = "\nHTTP STATUS CODE: " . $httpResponse['httpStatus'] . "\n";
        }
        if (isset($httpResponse['returnString'])) {
            // Format JSON to look nice on-screen
            if (\AutomateYourTests\Model\Generic\Strings::isJson($httpResponse['returnString'])) {
                $phpObject = json_decode($httpResponse['returnString']);
                $prettyHttpResponse = json_encode($phpObject, JSON_PRETTY_PRINT);
            } else {
                $prettyHttpResponse = $httpResponse['returnString'];
            }

            $this->partialLog[$sort] .= "\nHTTP RESPONSE: " . $prettyHttpResponse . "\n";
        }
    }

    /**
     * Give ultra-verbose feedback containing all <setup> or <teardown> return string ?
     */
    public function verboseExecutionResponse($nodeName, $node, $commandResponse)
    {
        $sort = $this->getNextSort();

        $this->partialLog[$sort] = '';

        if (isset($commandResponse['commandSource'])) {
            $this->partialLog[$sort] .= "\n" . strtoupper($nodeName) . " COMMAND: " . $commandResponse['commandSource'] . "\n";
        }
        if (isset($commandResponse['fileAndLine'])) {
            if (! strlen($commandResponse['fileAndLine'])) {
                $nodeSort = $this->getSort($node);
                $commandResponse['fileAndLine'] = $this->getLineNo($nodeSort);
            }

            $this->partialLog[$sort] .= "\n" . strtoupper($nodeName) . " LINE NUMBER: " . $commandResponse['fileAndLine'] . "\n";
        }
        if (isset($commandResponse['exitCode'])) {
            $this->partialLog[$sort] .= "\n" . strtoupper($nodeName) . " EXIT CODE: " . $commandResponse['exitCode'] . "\n";
        }
        if (isset($commandResponse['returnString'])) {
            $this->partialLog[$sort] .= "\n" . strtoupper($nodeName) . " RESPONSE: " . $commandResponse['returnString'] . " (may not always be complete, please run <" . $nodeName . "> command manually to debug)\n";
        }

        //$this->partialLog[$sort] .= "\nNOTE: The above command response string MAY NOT always be complete, so in case of failure please re-run the exact execution command manually in the console in order to receive a reliable output.\n";
    }

    /**
     * Make up a unique sort string for this SimpleXmlElement node
     */
    protected function getSort($domNode)
    {
        //$lineNo = '0';
        //$sort = $this->getNextSort();

        $domInfo = @\dom_import_simplexml($domNode);

        if ($domInfo instanceof \DOMElement) {
            $fileNo = $this->getFileNo($domInfo);
            $lineNo = $domInfo->getLineNo();

            $sort = (string) '"' . (($fileNo . '.' . $lineNo)) . '"';
            //$sort = (string) '"' . $lineNo . '"';
            //$sort = $lineNo;

            // Remember so that we can increase from here on next time
            if ($lineNo > 0) {
                $this->lastLineNo = $lineNo;
            }
        } else {
            $lineNo = 0;
        }

        if ($lineNo == 0) {
            $sort = $this->getNextSort();
        }

        //print "<pre>"; print_r($lineNo); print "</pre>\n";
        //print "<pre>"; print_r($this->lastLineNo); print "</pre>\n";
        //print "<pre>"; print_r($sort); print "</pre>\n";

        return $sort;
    }

    /**
     * Remember the order of all processed XML files (the main
     * test suite XML file as well all its xs:include files)
     * TODO: Move to ExtendedXMLElement
     */
    protected function getFileNo($domInfo)
    {
        // Default for inherited ??? lines
        $fileNo = 0;

        if ($domInfo instanceof \DOMElement) {
            $fileName = $domInfo->baseURI;

            if (! in_array($fileName, $this->fileNames)) {
                $this->fileNames[] = $fileName;
            }

            $fileNo = array_search($fileName, $this->fileNames);
        }

        $this->lastFileNo = $fileNo;

        return $fileNo;
    }

    /**
     * Increase float array index
     * TODO: Improve on this hack
     */
    protected function getNextSort($extraIntegerToAdd = 0)
    {
        $nextLineNo = ((float) $this->lastLineNo + $extraIntegerToAdd + $this->lastLineFraction);

        $sort = (string) '"' . (($this->lastFileNo . '.' . $nextLineNo)) . '"';
        //$sort = (string) '"' . $nextLineNo . '"';
        //$sort = $nextLineNo;

        // Remember so that we can increase from here on next time
        if ($nextLineNo > 0) {
            $this->lastLineNo = $nextLineNo;
        }

        return $sort;
    }

    /**
     * Get line number of where to find this SimpleXmlElement node
     */
    protected function getLineNo($sort)
    {
        $numericSort = str_replace('"', '', $sort);

        if (strpos($numericSort, '.') !== false) {
            $lineNo = substr($numericSort, strrpos($numericSort, '.') + 1);

            if (substr($lineNo, 0, 1) > 0) {
                $lineNo = intval($lineNo);
            } else {
                $currentLineNoLength = strlen(intval($this->lastLineNo));
                $lineNo = str_repeat($this->unknownLineNoSymbol, $currentLineNoLength);
            }
        }

        $fileName = $this->fileNames[$this->lastFileNo];

        if ($fileName) {
            $fileName = basename($fileName);
            $source = $lineNo . ': ' . $fileName;
            //$source = $fileName . ':' . $lineNo;
        } else {
            // TODO: State that file name can not be found!?
            $source = $lineNo;
        }

        return $source;
    }

    /**
     * Bring line numbers back into the order of how they appear in the original test suite XML file
     * (and consider log entries that are inherited or do not have any XML file equivalent)
     */
    public function sortLog()
    {
        \ksort($this->partialLog, SORT_NATURAL); // this sort flag is cos we have used strings as our array indexes, eg "4", "4.0001", "5", ...
    }

    /**
     *
     */
    public function addToLog()
    {
        // TODO: Make this dependant on the debugging flag
        //print "<pre>"; print_r($this->partialLog); print "</pre>\n";
        //die;

        $this->log .= implode('', $this->partialLog);
    }

    /**
     *
     */
    public function releaseLog()
    {
        $this->sortLog();

        echo implode('', $this->partialLog);
    }

    /**
     *
     */
    public function resetLog()
    {
        $this->partialLog = array();
    }

    /**
     * Return verbose log so that the user retains the flexibility to do with it whatever
     * they want (echo, save, ignore it, or print their own toilet paper from it)
     */
    public function getLog()
    {
        return $this->log;
    }

    /**
     * Return verbose log for interactive mode
     */
    public function getInteractiveLog()
    {
        $this->log  = "\n---------------------------------------------------------------------------------------------------\n\n";
        $this->log .= 'NOTE: THE TEST SUITE WAS RUN INTERACTIVELY IN DEBUG MODE, SO THE GETLOG() CALL RESULTED IN NOTHING';

        return $this->log;
    }

    /**
     *
     */
    /* public function set($newLogString)
    {
        $this->log = $newLogString;
    } */

    /**
     * Get TO-FAIL result up to this point
     */
    public function toFailureSummary($testsuite, $extraLogInfo = '')
    {
        $this->extraLogInfo = $extraLogInfo;
    }

    /**
     * Get INTERACTIVE result up to this point
     */
    public function buildInteractiveSummary($testsuite, $extraLogInfo = '')
    {
        $sort = $this->getNextSort();

        $this->extraLogInfo = $extraLogInfo;

        $this->partialLog[$sort] = $this->separator();
        $this->partialLog[$sort] = $this->getSummary($testsuite, $extraLogInfo);
        $this->partialLog[$sort] = "\n";
    }

    /**
     * Get final result sentence for interactive mode
     */
    public function getInteractiveSummary($testsuite)
    {
        $summary  = "\n\n---------------------------------------------------------------------------------------------------\n\n";
        $summary .= 'NOTE: THE TEST SUITE WAS RUN INTERACTIVELY IN DEBUG MODE, SO THE GETSUMMARY() CALL RESULTED IN NOTHING';

        return $summary;
    }

    /**
     * Get final result sentence
     */
    public function getSummary($testsuite)
    {
        // The interactive/to-fail modes inject summit here at run-time
        if ($this->extraLogInfo) {
            $extraLogInfo = $this->extraLogInfo;
        } else {
            $extraLogInfo = '';
        }

        $fullConfigXmlPath = \realpath($testsuite);

        $userFailureCount = $this->model->counter->howManyUsers('failure');
        $userAllCount = $this->model->counter->howManyUsers('all');
        $setupFailureCount = $this->model->counter->howManySetups('failure');
        $setupAllCount = $this->model->counter->howManySetups('all');
        $teardownFailureCount = $this->model->counter->howManyTeardowns('failure');
        $teardownAllCount = $this->model->counter->howManyTeardowns('all');

        if ($userFailureCount > 0) {
            $summary = 'OVERALL FAILURE:' . $extraLogInfo . ' ' . $userFailureCount . ' OUT OF ' . $userAllCount . ' <USER> LOGINS HAVE FAILED AND FURTHER TESTING HAS BEEN SUSPENDED IN TEST SUITE ' . $fullConfigXmlPath . '.' . $this->getScriptTime();
        } elseif ($setupFailureCount > 0) {
            $summary = 'OVERALL FAILURE:' . $extraLogInfo . ' ' . $setupFailureCount . ' OUT OF ' . $setupAllCount . ' <SETUP> EXECUTION COMMANDS HAVE FAILED AND FURTHER TESTING HAS BEEN SUSPENDED IN TEST SUITE ' . $fullConfigXmlPath . '.' . $this->getScriptTime();
        } elseif ($teardownFailureCount > 0) {
            $summary = 'OVERALL FAILURE:' . $extraLogInfo . ' ' . $teardownFailureCount . ' OUT OF ' . $teardownAllCount . ' <TEARDOWN> EXECUTION COMMANDS HAVE FAILED AND FURTHER TESTING HAS BEEN SUSPENDED IN TEST SUITE ' . $fullConfigXmlPath . '.' . $this->getScriptTime();
        } else {
            $successCount = $this->allCount - $this->failureCount;

            if ($this->failureCount == 0 && $successCount > 1) {
                $summary = 'OVERALL PASS:' . $extraLogInfo . ' THE ' . $successCount . ' TEST STEPS HAVE PASSED IN TEST SUITE ' . $fullConfigXmlPath . '.' . $this->getScriptTime();
            } elseif ($this->failureCount == 0) {
                $summary = 'OVERALL PASS:' . $extraLogInfo . ' THE ' . $successCount . ' TEST STEP HAS PASSED IN TEST SUITE ' . $fullConfigXmlPath . '.' . $this->getScriptTime();
                //$summary = 'OVERALL PASS:' . $extraLogInfo . ' THE ' . $successCount . ' TEST STEP HAS PASSED IN TEST SUITE 1_hello_world.xml.' . $this->getScriptTime();
            } else {
                $summary = 'OVERALL FAILURE:' . $extraLogInfo . ' ' . $this->failureCount . ' OUT OF ' . $this->allCount . ' TEST STEPS HAVE FAILED IN TEST SUITE ' . $fullConfigXmlPath . $this->getScriptTime();
            }
        }

        return $summary;
    }

    /**
     * Remember and format the start and end time of how long the script ran
     */
    protected function getScriptTime()
    {
        $this->scriptEndDate = new \DateTime();
        $prettyEndDate = $this->scriptEndDate->format('D m/d/Y H:i:s');
        $prettyEndDate = \strtoupper($prettyEndDate);
        $scriptRunTime = \AutomateYourTests\Model\Generic\Datetimes::calculateDifference($this->scriptStartDate, $this->scriptEndDate);

        if ($scriptRunTime->h == 0 && $scriptRunTime->i == 0) {
            $prettyScriptRunTime = sprintf('%d SECS', $scriptRunTime->s);
        } elseif ($scriptRunTime->h == 0) {
            $prettyScriptRunTime = sprintf('%d MINS, %d SECS', $scriptRunTime->i, $scriptRunTime->s);
        } else {
            $prettyScriptRunTime = sprintf('%d HRS, %d MINS, %d SECS', $scriptRunTime->h, $scriptRunTime->i, $scriptRunTime->s);
        }

        $timeInfo = ' THIS TOOK ' . $prettyScriptRunTime . ' TO COMPLETE AND FINISHED ' . $prettyEndDate . ".\n";

        return $timeInfo;
    }

    /**
     * Set config class var
     */
    public function setConfig($config = array())
    {
        $this->config = $config;
    }
}
