<?php
/**
 * Use this PHP file to run your own custom test suites using the AutomateYourTests software tool.
 *
 * The following example runs small proof-of-concept test scenario on a dedicated Wordpress demo web application that I
 * have set up especially for this purpose. Each XML title="" attribute below documents the exact feature of the
 * testing software that this demo tries to teach. Enough words ... make sure you have Internet access, then simply
 * change into the directory where you extracted the “AutomateYourTests” ZIP and execute one of the following
 * commands and see what happens ...
 *
 * @example     php run.php testsuite="Examples/1_3_hello_world.xml"
 * @example     php run.php testsuite="Examples/1_3_hello_world.xml" from="20" to="25"
 * @example     php run.php testsuite="Examples/1_3_hello_world.xml" verbose="failure" to="failure"
 *
 * @link        http://automateyourtests.com
 * @version     1.2.2
 * @author      Marc Daniel Dammann <howto@automateyourtests.com>
 * @copyright   2016 Marc Daniel Dammann
 * @license     GPL-3.0 <https://opensource.org/licenses/GPL-3.0>
 *
 * Copyright 2016 Marc Daniel Dammann
 * This file is part of AutomateYourTests. AutomateYourTests is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. AutomateYourTests is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with AutomateYourTests. If not, see http://www.gnu.org/licenses/.
 */

// Maximum time you expect your test suites to run (in seconds)
set_time_limit(300);

// Be safe
if (version_compare(PHP_VERSION, '5.4.0') < 0) {
    echo('Your PHP version is ' . PHP_VERSION . ', but AutomateYourTests requires at least 5.4.0');
} else {
    try {
        // The test software
        require('Controller/Base.php');

        // The $argv contains the script arguments. The @example comment
        // in the file header above lists what arguments you can pass.
        $tests = new AutomateYourTests\Controller\Base($argv);

        // Run the tests in your specified testsuite="filename.xml"
        $tests->run();

        // Up to you what you want to do with this test result log:
        // Echo it, save it, ignore it, or print your own toilet paper from it!
        echo $tests->getLog();

        // Want to email yourself a test result summary every night!?
        echo $tests->getLog('summary') . "\n\n";

        // Exit code (0 = if all the steps in the test suite succeed / 1 = if at least one test step fails).
        // That way you can hook up your tests to your Version Control, Continuous Integration system, etc.
        exit($tests->getExitCode());
    } catch (Exception $e) {
        // Any problems in running the test suite bubble up to here. Handle them to your own liking.
        echo "Error: " . $e->getMessage() . "\n"; // $e->getTraceAsString();
    }
}

// Exit with failure code, just in case it ever gets to this line
exit(1);
